
(* Interface file for interpret.ml. *)

exception Error of string

module VarType : sig type t = string val compare : 'a -> 'a -> int end

module Env :
  sig
    type key = VarType.t
    and 'a t = 'a Map.Make(VarType).t
    val empty : 'a t
    val add : key -> 'a -> 'a t -> 'a t
    val find : key -> 'a t -> 'a
    val remove : key -> 'a t -> 'a t
    val mem : key -> 'a t -> bool
    val iter : (key -> 'a -> unit) -> 'a t -> unit
    val map : ('a -> 'b) -> 'a t -> 'b t
    val mapi : (key -> 'a -> 'b) -> 'a t -> 'b t
    val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
  end

type rtv =
    BoolV of bool
  | IntV of int
  | EmptyV
  | VoidV
  | PairV of rtv ref * rtv ref
  | ClosureV of string list * Absyn.exp * env
and env = rtv ref Env.t

val eval : Absyn.exp -> rtv

val print : rtv -> unit

