(* File main.ml *)
(* Amr Sabry *)

let rec main () = 
  let lexbuf = Lexing.from_channel stdin in
  Format.open_box 0;
  try loop lexbuf
  with 
    Mlexer.Eof -> Format.print_flush(); exit 0
  | Parsing.Parse_error -> Format.print_string "Parse error" ; loop lexbuf

and loop lexbuf = 
  while true do
    begin
      let e = Mparser.main Mlexer.token lexbuf in
      Format.print_string "Input expression:";
      Format.print_space();
      Absyn.print e;
      Format.print_newline();
      (try 
        let v = Interpret.eval e in
        Format.print_string "Value (imperative interpreter):";
        Format.print_space();
        Interpret.print v
      with Interpret.Error s -> 
        Format.print_string ("Imperative interpreter \n" ^ s ^ "\n"));
      (try 
        let (v,st) = Sinterpret.eval e in
        Format.print_string "Value (store-passing interpreter):";
        Format.print_space();
        Sinterpret.print v st
      with Sinterpret.Error s -> 
        Format.print_string ("Store-passing interpreter \n" ^ s ^ "\n"));
      Format.print_string "\n******************";
      Format.print_newline()
    end
  done
  
let _ = main ()

