(* lexer.mll for the pure subset of Scheme *)
(* Amr Sabry *)

{
(* Header *)
open Mparser
exception Eof
}

let digit = ['0'-'9']
let num = '-'? digit+
let letter = ['a'-'z' 'A'-'Z']
let id = '_' | (letter (letter | digit | ['-' '_' '!' '?'])*)

rule token = parse
     [' ' '\t' '\n']  { token lexbuf }   (* skip blanks *)
  |  ';'[^'\n']*'\n'  { token lexbuf }   (* skip comments *)
  |  "procedure?"     { PROCEDUREP }
  |  "set-car!"       { SETCAR }
  |  "set-cdr!"       { SETCDR }
  |  "integer?"       { INTEGERP }
  |  "boolean?"       { BOOLEAN }
  |  "letrec"         { LETREC }
  |  "lambda"         { LAMBDA }
  |  "quote"          { QUOTE }
  |  "begin"          { BEGIN }
  |  "zero?"          { ZEROP }
  |  "null?"          { NULLP }
  |  "pair?"          { PAIRP }
  |  "void"           { VOID }
  |  "set!"           { SET }
  |  "sub1"           { SUB1 }
  |  "cons"           { CONS }
  |  "add1"           { ADD1 }
  |  "eq?"            { EQP }
  |  "car"            { CAR }
  |  "cdr"            { CDR }
  |  "not"            { NOT }
  |  "let"            { LET }
  |  "if"             { IF }
  |  "#t"             { TRUE }
  |  "#f"             { FALSE }
  |  "<="             { LEQ }
  |  '\''             { QUOTECHAR }
  |  '('              { LPAREN }
  |  ')'              { RPAREN }
  |  '['              { LPAREN }
  |  ']'              { RPAREN }
  |  '-'              { MINUS }
  |  '.'              { DOT }
  |  '*'              { TIMES }
  |  '+'              { PLUS }
  |  '<'              { LESS }
  |  '='              { EQ }
  |  num              { INT(int_of_string(Lexing.lexeme lexbuf)) }
  |  id               { ID(Lexing.lexeme lexbuf) }
  |  eof              { raise Eof }

{
(* Trailer *)
} 
