/* File mparser.mly for the pure subset of Scheme */
/* Amr Sabry */

%token QUOTE IF BEGIN LET LETREC LAMBDA
%token QUOTECHAR LPAREN RPAREN DOT 
%token TRUE FALSE 
%token <int> INT
%token <string>ID
%token MINUS TIMES PLUS LESS LEQ EQ ADD1 BOOLEAN CAR CDR CONS INTEGERP
%token NOT NULLP PAIRP PROCEDUREP SUB1 VOID ZEROP
%token EQP SETCAR SETCDR SET

%start main
%type <Absyn.exp> main 
%type <(string * Absyn.exp) list> declstar

%%

main: 
          exp { $1 }
;

exp: 
          TRUE 
                { Absyn.ConstantE (Absyn.BoolC true) }
        | FALSE 
                { Absyn.ConstantE (Absyn.BoolC false) }
        | INT 
                { Absyn.ConstantE (Absyn.IntC $1) }
        | ID { Absyn.VarE $1 }
        | LPAREN QUOTE datum RPAREN { Absyn.QuoteE $3 }
        | QUOTECHAR datum { Absyn.QuoteE $2 }
        | LPAREN SET ID exp RPAREN { Absyn.SetE($3,$4) }
        | LPAREN IF exp exp RPAREN { Absyn.OneifE($3,$4) }
        | LPAREN IF exp exp exp RPAREN { Absyn.IfE($3,$4,$5) }
        | LPAREN BEGIN exp expstar RPAREN { Absyn.BeginE($3::$4) }
        | LPAREN LET LPAREN declstar RPAREN exp RPAREN 
                { let d = List.split $4 in
                  Absyn.LetE(fst d, snd d, $6) }
        | LPAREN LETREC LPAREN declstar RPAREN exp RPAREN 
                { let d = List.split $4 in
                  Absyn.LetrecE(fst d, snd d, $6) } 
        | LPAREN LAMBDA LPAREN idstar RPAREN exp RPAREN
                { Absyn.LambdaE($4, $6) }
        | LPAREN prim expstar RPAREN 
                { Absyn.PrimappE($2,$3) }
        | LPAREN exp expstar RPAREN { Absyn.AppE($2,$3) }
;

datum: 
          TRUE { Absyn.ConstantD (Absyn.BoolC true) }
        | FALSE { Absyn.ConstantD (Absyn.BoolC false) }
        | INT { Absyn.ConstantD (Absyn.IntC $1) }
        | LPAREN RPAREN { Absyn.EmptyListD }
;

decl: 
        LPAREN ID exp RPAREN { ($2,$3) }

expstar:
          { [] }
        | exp expstar { $1::$2 }
;

declstar: 
          { [] }
        | decl declstar { $1::$2 }
;

idstar: 
          { [] }
        | ID idstar { $1::$2 }
;

prim: 
          MINUS { Absyn.Minus } 
        | TIMES { Absyn.Times }
        | PLUS { Absyn.Plus }
        | LESS { Absyn.Less } 
        | LEQ { Absyn.Leq } 
        | EQ { Absyn.Eq } 
        | ADD1 { Absyn.Add1 }
        | BOOLEAN { Absyn.Booleanp }
        | CAR { Absyn.Car }
        | CDR { Absyn.Cdr } 
        | CONS { Absyn.Cons }
        | INTEGERP { Absyn.Integerp }
        | NOT { Absyn.Not }
        | NULLP { Absyn.Nullp }
        | PAIRP { Absyn.Pairp }
        | PROCEDUREP { Absyn.Procedurep }
        | SUB1 { Absyn.Sub1 }
        | VOID { Absyn.Void }
        | ZEROP { Absyn.Zerop }
        | EQP { Absyn.Eqp }
        | SETCAR { Absyn.Setcar }
        | SETCDR { Absyn.Setcdr }

/**************************************************************************/