exception Error of string

module VarType : sig type t = string val compare : 'a -> 'a -> int end

module Env :
  sig
    type key = VarType.t
    and 'a t = 'a Map.Make(VarType).t
    val empty : 'a t
    val add : key -> 'a -> 'a t -> 'a t
    val find : key -> 'a t -> 'a
    val remove : key -> 'a t -> 'a t
    val mem : key -> 'a t -> bool
    val iter : (key -> 'a -> unit) -> 'a t -> unit
    val map : ('a -> 'b) -> 'a t -> 'b t
    val mapi : (key -> 'a -> 'b) -> 'a t -> 'b t
    val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
  end

type location = int

and rtv =
    BoolV of bool
  | IntV of int
  | EmptyV
  | VoidV
  | PairV of location * location
  | ClosureV of string list * Absyn.exp * env

and env = location Env.t

module LocType : sig type t = location val compare : 'a -> 'a -> int end

module StoreM :
  sig
    type key = LocType.t
    and 'a t = 'a Map.Make(LocType).t
    val empty : 'a t
    val add : key -> 'a -> 'a t -> 'a t
    val find : key -> 'a t -> 'a
    val remove : key -> 'a t -> 'a t
    val mem : key -> 'a t -> bool
    val iter : (key -> 'a -> unit) -> 'a t -> unit
    val map : ('a -> 'b) -> 'a t -> 'b t
    val mapi : (key -> 'a -> 'b) -> 'a t -> 'b t
    val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
  end

module type STORE =
  sig
    type store
    val empty : store
    val alloc : rtv -> store -> location * store
    val allocN : int -> store -> location list * store
    val update : location -> rtv -> store -> store
    val find : location -> store -> rtv
  end

module Store : STORE

val eval : Absyn.exp -> rtv * Store.store

val print : rtv -> Store.store -> unit
